@@ -132,7 +132,15 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "liblavplay.c"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+
+#include "liblavplay.h"
 #include "mjpeg_logging.h"
 #include <mpegconsts.h>
 #include <mpegtimecode.h>
@@ -141,6 +149,9 @@
 #include <getopt.h>
 #endif
 #include <fcntl.h>
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
 
 #define LAVPLAY_VSTR "lavplay" LAVPLAY_VERSION  /* Expected version info */
 
