@@ -1,19 +1,20 @@
 /*
- * $Id: pgmtoy4m.c,v 1.10 2003/11/23 06:08:03 sms00 Exp $
+ * $Id: pgmtoy4m.c,v 1.14 2003/12/17 16:54:04 sms00 Exp $
  *
  * pgmtoy4m converts the PGM output of "mpeg2dec -o pgmpipe" to YUV4MPEG2 on
  * stdout.
  *
- * Note: mpeg2dec uses a perversion of the PGM format - they're reall not
+ * Note: mpeg2dec uses a perversion of the PGM format - they're really not
  * "Grey Maps" but rather a catenation of the 420P data (commonly called
  * "YUV").    The type is P5 ("raw") and the number of rows is really
- * the total of the Y', Cr and Cb heights.   The U and V data is "joined"
- * together - you have 1 row of U and 1 row of V per "row" of PGM data.
+ * the total of the Y', Cb and Cr heights.   The Cb and Cr data is "joined"
+ * together.  After the Y' rows you have 1 row of Cb and 1 row of Cr per 
+ * "row" of PGM data.
  *
  * NOTE: You MAY need to know the field order (top or bottom field first),
  *	sample aspect ratio and frame rate because the PGM format makes
  *	none of that information available!  There are defaults provided
- *	that hopefully will do the right thing in the common cases.  The
+ *	that hopefully will do the right thing in the common cases.
  *
  *	The defaults provided are: top field first, NTSC rate of 30000/1001
  *	frames/second, and a sample aspect of 10:11.
@@ -32,9 +33,7 @@
 
 #include "yuv4mpeg.h"
 
-extern	char	*__progname;
-
-static	void	usage(void);
+static	void	usage(char *progname);
 static	int	getint(int);
 
 #define	P5MAGIC	(('P' * 256) + '5')
@@ -98,7 +97,7 @@ main(int argc, char **argv)
 						ilace = Y4M_ILACE_BOTTOM_FIRST;
 						break;
 					default:
-						usage();
+						usage(argv[0]);
 					}
 				break;
 			case	'v':
@@ -121,7 +120,7 @@ main(int argc, char **argv)
 			case	'h':
 			case	'?':
 			default:
-				usage();
+				usage(argv[0]);
 				/* NOTREACHED */
 			}
 		}
@@ -255,12 +254,12 @@ main(int argc, char **argv)
 	}
 
 static void
-usage(void)
+usage(char *progname)
 	{
 
 	fprintf(stderr, "%s usage: [-v n] [-i t|b|p] [-a sample aspect] [-r rate]\n",
-		__progname);
-	fprintf(stderr, "%s\taspect and rate in ratio form: -a  10:11 and -r 30000:1001 or -r 25:1 for example\n", __progname);
+		progname);
+	fprintf(stderr, "%s\taspect and rate in ratio form: -a  10:11 and -r 30000:1001 or -r 25:1 for example\n", progname);
 	exit(0);
 	}
 
