@@ -140,7 +140,8 @@ main(int argc, char *argv[])
 		    SS_H = 4;
 		    SS_V = 1;
 		  } 
-		else if (!strcmp("420", tag))
+		else if (!strcmp(tag, "420") || !strcmp(tag, "420MPEG2") || 
+                         !strcmp(tag, "420PALDV") || !strcmp(tag,"420JPEG"))
 		  {
 		    SS_H = 2;
 		    SS_V = 2;
