@@ -31,8 +31,6 @@
 #include <string.h>
 #include <errno.h>
 #include "simd.h"
-#include "mpegconsts.h"
-#include "yuv4mpeg.h"
 #include "mpeg2encoder.hh"
 
 PictureReader::PictureReader( EncoderParams &_encparams ) :
@@ -442,126 +440,6 @@ int PictureReader::FrameLumMean( int num_frame )
 }
 
 
-Y4MPipeReader::Y4MPipeReader( EncoderParams &encparams, int istrm_fd ) :
-    PictureReader( encparams ),
-    pipe_fd( istrm_fd )
-{
-}
-
-/****************************************
- *
- * Initialise the reader and return the parameter of the video stream
- * to be encoded.
- *
- * WARNING: This routine must run before encoder parameters are defined.
- * TODO: Reader should be constructed before EncoderParams and this
- * routine invoked from EncoderParams constructor...
- *
- *****************************************/
-
-void Y4MPipeReader::StreamPictureParams( MPEG2EncInVidParams &strm )
-{
-   int n;
-   y4m_ratio_t sar;
-   y4m_stream_info_t si;
-
-   y4m_init_stream_info (&si);  
-   if ((n = y4m_read_stream_header (pipe_fd, &si)) != Y4M_OK) {
-       mjpeg_log( LOG_ERROR, 
-                  "Could not read YUV4MPEG2 header: %s!",
-                  y4m_strerr(n));
-      exit (1);
-   }
-
-   strm.horizontal_size = y4m_si_get_width(&si);
-   strm.vertical_size = y4m_si_get_height(&si);
-   strm.frame_rate_code = mpeg_framerate_code(y4m_si_get_framerate(&si));
-   strm.interlacing_code = y4m_si_get_interlace(&si);
-
-   /* Deduce MPEG aspect ratio from stream's frame size and SAR...
-      (always as an MPEG-2 code; that's what caller expects). */
-   sar = y4m_si_get_sampleaspect(&si);
-   strm.aspect_ratio_code = 
-       mpeg_guess_mpeg_aspect_code(2, sar, 
-                                   strm.horizontal_size, 
-                                   strm.vertical_size);
-   if(strm.horizontal_size <= 0)
-   {
-       mjpeg_error_exit1("Horizontal size from input stream illegal");
-   }
-   if(strm.vertical_size <= 0)
-   {
-       mjpeg_error("Vertical size from input stream illegal");
-   }
-}
-
-/*****************************
- *
- * LoadFrame - pull in the image data planes from pipe (planar YUV420)
- *
- * RETURN: true iff EOF or ERROR
- *
- ****************************/
-
-bool Y4MPipeReader::LoadFrame( )
-{
-   y4m_frame_info_t fi;
-   int h,v,y;
-   y4m_init_frame_info (&fi);
-   int buffer_slot = frames_read % input_imgs_buf_size;
-
-
-   if ((y = y4m_read_frame_header (pipe_fd, &fi)) != Y4M_OK) 
-   {
-       if( y != Y4M_ERR_EOF )
-           mjpeg_log (LOG_WARN, 
-                      "Error reading frame header (%d): code%s!", 
-                      frames_read,
-                      y4m_strerr (y));
-       return true;
-      }
-      
-   v = encparams.vertical_size;
-   h = encparams.horizontal_size;
-   int i;
-   for(i=0;i<v;i++)
-   {
-       if( PipeRead(input_imgs_buf[buffer_slot][0]+i*encparams.phy_width,h)!=h)
-           return true;
-   }
-   lum_mean[buffer_slot] = LumMean(input_imgs_buf[buffer_slot][0] );
-   v = encparams.vertical_size/2;
-   h = encparams.horizontal_size/2;
-   for(i=0;i<v;i++)
-   {
-       if(PipeRead(input_imgs_buf[buffer_slot][1]+i*encparams.phy_chrom_width,h)!=h)
-           return true;
-   }
-   for(i=0;i<v;i++)
-   {
-       if(PipeRead(input_imgs_buf[buffer_slot][2]+i*encparams.phy_chrom_width,h)!=h)
-           return true;
-   }
-   return false;
-}
-
-
-
-int Y4MPipeReader::PipeRead(uint8_t *buf, int len)
-{
-   int n, r;
-
-   r = 0;
-
-   while(r<len)
-   {
-      n = read(pipe_fd,buf+r,len-r);
-      if(n==0) return r;
-      r += n;
-   }
-   return r;
-}
-
 
 /* 
  * Local variables:
