@@ -1,5 +1,6 @@
-/* putseq.c, sequence level routines                                        */
+/* putseq.c MPEG1/2 Sequence encoding loop */
 
+//#define SEQ_DEBUG 1
 /* (C) Andrew Stevens 2000, 2001
  *  This file is free software; you can redistribute it
  *  and/or modify it under the terms of the GNU General Public License
@@ -41,7 +42,7 @@
  * design.
  *
  */
-
+
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -49,167 +50,43 @@
 #include <errno.h>
 #include "mjpeg_types.h"
 #include "mjpeg_logging.h"
-#include "global.h"
-#include "motionsearch.h"
+#include "simd.h"
+#include "mpeg2syntaxcodes.h"
+#include "mpeg2encoder.hh"
+#include "picturereader.hh"
+#include "mpeg2coder.hh"
+#include "seqencoder.hh"
 #include "ratectl.hh"
+#include "tables.h"
+
+
+
+SeqEncoder::SeqEncoder( EncoderParams &_encparams,
+                        PictureReader &_reader,
+                        Quantizer &_quantizer,
+                        ElemStrmWriter &_writer,
+                        MPEG2Coder &_coder,
+                        RateCtl    &_ratecontroller ) :
+    encparams( _encparams ),
+    reader( _reader ),
+    quantizer( _quantizer ),
+    writer( _writer ),
+    coder( _coder ),
+    ratecontroller( _ratecontroller )
 
-static RateCtl *bitrate_controller = 0;
-static void set_pic_params( int decode,
-							int b_index,
-							Picture *picture )
 {
-	picture->decode = decode;
-	picture->dc_prec = opt_dc_prec;
-	picture->secondfield = false;
-	picture->ipflag = 0;
-
-		
-	/* Handle picture structure... */
-	if( opt_fieldpic )
-	{
-		picture->pict_struct = opt_topfirst ? TOP_FIELD : BOTTOM_FIELD;
-		picture->topfirst = 0;
-		picture->repeatfirst = 0;
-	}
-
-	/* Handle 3:2 pulldown frame pictures */
-	else if( opt_pulldown_32 )
-	{
-		picture->pict_struct = FRAME_PICTURE;
-		switch( picture->present % 4 )
-		{
-		case 0 :
-			picture->repeatfirst = 1;
-			picture->topfirst = opt_topfirst;			
-			break;
-		case 1 :
-			picture->repeatfirst = 0;
-			picture->topfirst = !opt_topfirst;
-			break;
-		case 2 :
-			picture->repeatfirst = 1;
-			picture->topfirst = !opt_topfirst;
-			break;
-		case 3 :
-			picture->repeatfirst = 0;
-			picture->topfirst = opt_topfirst;
-			break;
-		}
-	}
-	
-	/* Handle ordinary frame pictures */
-	else
-
-	{
-		picture->pict_struct = FRAME_PICTURE;
-		picture->repeatfirst = 0;
-		picture->topfirst = opt_topfirst;
-	}
-
-
-	switch ( picture->pict_type )
-	{
-	case I_TYPE :
-		picture->forw_hor_f_code = 15;
-		picture->forw_vert_f_code = 15;
-		picture->back_hor_f_code = 15;
-		picture->back_vert_f_code = 15;
-		picture->sxf = opt_motion_data[0].sxf;
-		picture->syf = opt_motion_data[0].syf;
-		break;
-	case P_TYPE :
-		picture->forw_hor_f_code = opt_motion_data[0].forw_hor_f_code;
-		picture->forw_vert_f_code = opt_motion_data[0].forw_vert_f_code;
-		picture->back_hor_f_code = 15;
-		picture->back_vert_f_code = 15;
-		picture->sxf = opt_motion_data[0].sxf;
-		picture->syf = opt_motion_data[0].syf;
-		break;
-	case B_TYPE :
-		picture->forw_hor_f_code = opt_motion_data[b_index].forw_hor_f_code;
-		picture->forw_vert_f_code = opt_motion_data[b_index].forw_vert_f_code;
-		picture->back_hor_f_code = opt_motion_data[b_index].back_hor_f_code;
-		picture->back_vert_f_code = opt_motion_data[b_index].back_vert_f_code;
-		picture->sxf = opt_motion_data[b_index].sxf;
-		picture->syf = opt_motion_data[b_index].syf;
-		picture->sxb = opt_motion_data[b_index].sxb;
-		picture->syb = opt_motion_data[b_index].syb;
-
-		break;
-	}
-
-	/* We currently don't support frame-only DCT/Motion Est.  for non
-	   progressive frames */
-	picture->prog_frame = opt_frame_pred_dct_tab[picture->pict_type-1];
-	picture->frame_pred_dct = opt_frame_pred_dct_tab[picture->pict_type-1];
-	picture->q_scale_type = opt_qscale_tab[picture->pict_type-1];
-	picture->intravlc = opt_intravlc_tab[picture->pict_type-1];
-	picture->altscan = opt_altscan_tab[picture->pict_type-1];
-
-#ifdef OUTPUT_STAT
-	fprintf(statfile,"\nFrame %d (#%d in display order):\n",decode,display);
-	fprintf(statfile," picture_type=%c\n",pict_type_char[picture->pict_type]);
-	fprintf(statfile," temporal_reference=%d\n",picture->temp_ref);
-	fprintf(statfile," frame_pred_frame_dct=%d\n",picture->frame_pred_dct);
-	fprintf(statfile," q_scale_type=%d\n",picture->q_scale_type);
-	fprintf(statfile," intra_vlc_format=%d\n",picture->intravlc);
-	fprintf(statfile," alternate_scan=%d\n",picture->altscan);
-
-	if (picture->pict_type!=I_TYPE)
-	{
-		fprintf(statfile," forward search window: %d...%d / %d...%d\n",
-				-sxf,sxf,-syf,syf);
-		fprintf(statfile," forward vector range: %d...%d.5 / %d...%d.5\n",
-				-(4<<picture->forw_hor_f_code),(4<<picture->forw_hor_f_code)-1,
-				-(4<<picture->forw_vert_f_code),(4<<picture->forw_vert_f_code)-1);
-	}
-
-	if (picture->pict_type==B_TYPE)
-	{
-		fprintf(statfile," backward search window: %d...%d / %d...%d\n",
-				-sxb,sxb,-syb,syb);
-		fprintf(statfile," backward vector range: %d...%d.5 / %d...%d.5\n",
-				-(4<<picture->back_hor_f_code),(4<<picture->back_hor_f_code)-1,
-				-(4<<picture->back_vert_f_code),(4<<picture->back_vert_f_code)-1);
-	}
-#endif
-
-
+    mp_semaphore_init( &worker_available, 0 );
+    mp_semaphore_init( &picture_available, 0 );
+    mp_semaphore_init( &picture_started, 0 );
 }
 
-/*
- * Adjust picture parameters for the second field in a pair of field
- * pictures.
- *
- */
-
-static void set_2nd_field_params(Picture *picture)
+SeqEncoder::~SeqEncoder()
 {
-	picture->secondfield = true;
-    picture->gop_start = false;
-	if( picture->pict_struct == TOP_FIELD )
-		picture->pict_struct =  BOTTOM_FIELD;
-	else
-		picture->pict_struct =  TOP_FIELD;
-	
-	if( picture->pict_type == I_TYPE )
-	{
-		picture->ipflag = 1;
-		picture->pict_type = P_TYPE;
-		
-		picture->forw_hor_f_code = opt_motion_data[0].forw_hor_f_code;
-		picture->forw_vert_f_code = opt_motion_data[0].forw_vert_f_code;
-		picture->back_hor_f_code = 15;
-		picture->back_vert_f_code = 15;
-		picture->sxf = opt_motion_data[0].sxf;
-		picture->syf = opt_motion_data[0].syf;	
-	}
 }
 
-
 /************************************
  *
- * gop_end_frame - Finds a sensible spot for a GOP to end based on
+ * FindGopLength - Finds a sensible spot for a GOP to end based on
  * the mean luminance strategy developed in Dirk Farin's sampeg-2
  *
  * Basically it looks for a frame whose mean luminance is above
@@ -232,34 +109,33 @@ static void set_2nd_field_params(Picture *picture)
 
 #define SCENE_CHANGE_THRESHOLD 4
 
-static int find_gop_length( int gop_start_frame, 
-							int I_frame_temp_ref,
-							int gop_min_len, int gop_max_len,
-							int min_b_grp)
+int SeqEncoder::FindGopLength( int gop_start_frame, 
+                               int I_frame_temp_ref,
+                               int gop_min_len, int gop_max_len,
+                               int min_b_grp)
 {
 	int i,j;
-    /*
-      Checking the luminance mean of the frame also updates
-      istrm_nframes if the number of frames in the stream if the end
-      of stream has is reached before this frame...
-    */
     int max_change = 0;
     int gop_len;
-	int cur_lum_mean = 
-		frame_lum_mean( gop_start_frame+gop_min_len-min_b_grp+I_frame_temp_ref );
 	int pred_lum_mean;
 
     if( gop_min_len >= gop_max_len )
         gop_len = gop_max_len;
     else
     {
+        int cur_lum_mean = 
+            reader.FrameLumMean( gop_start_frame+gop_min_len-min_b_grp+I_frame_temp_ref );
+
         /* Search forwards from min gop length for I-frame candidate
            which has the largest change in mean luminance.
+           BUGBUGBUG: The frame ring buffer size must be large enough to
+           accomodate this read-ahead without losing frames currently being
+           encoded!!!
         */
         gop_len = 0;
         for( i = gop_min_len; i <= gop_max_len; i += min_b_grp )
         {
-            pred_lum_mean = frame_lum_mean( gop_start_frame+i+I_frame_temp_ref);
+            pred_lum_mean = reader.FrameLumMean( gop_start_frame+i+I_frame_temp_ref);
             if( abs(cur_lum_mean-pred_lum_mean ) >= SCENE_CHANGE_THRESHOLD 
                 && abs(cur_lum_mean-pred_lum_mean ) > max_change )
             {
@@ -281,7 +157,7 @@ static int find_gop_length( int gop_start_frame,
             for( j = gop_max_len+min_b_grp; j < gop_max_len+gop_min_len; j += min_b_grp )
             {
                 cur_lum_mean = 
-                    frame_lum_mean( gop_start_frame+j+I_frame_temp_ref );
+                    reader.FrameLumMean( gop_start_frame+j+I_frame_temp_ref );
                 if( abs(cur_lum_mean-pred_lum_mean ) >= SCENE_CHANGE_THRESHOLD
                     &&  abs(cur_lum_mean-pred_lum_mean > max_change )
                     )
@@ -329,6 +205,7 @@ static int find_gop_length( int gop_start_frame,
         }
     }
 	/* last GOP may contain less frames! */
+    int istrm_nframes = reader.NumberOfFrames();
 	if (gop_len > istrm_nframes-gop_start_frame)
 		gop_len = istrm_nframes-gop_start_frame;
 	mjpeg_info( "GOP start (%d frames)", gop_len);
@@ -338,31 +215,11 @@ static int find_gop_length( int gop_start_frame,
 
 
 
-struct _stream_state 
-{
-	int i;						/* Index in current sequence */
-	int g;						/* Index in current GOP */
-	int b;						/* Index in current B frame group */
-	int seq_start_frame;		/* Index start current sequence in
-								   input stream */
-	int gop_start_frame;		/* Index start current gop in input stream */
-	int gop_length;			/* Length of current gop */
-	int bigrp_length;			/* Length of current B-frame group */
-	int bs_short;				/* Number of B frame GOP is short of
-								   having M-1 B's for each I/P frame
-								*/
-	int np;					/* P frames in current GOP */
-	int nb;					/* B frames in current GOP */
-	double next_b_drop;		/* When next B frame drop is due in GOP */
-	bool new_seq;				/* Current GOP/frame starts new sequence */
-    bool closed_gop;            /* Current GOP is closed */
-	uint64_t next_split_point;
-	uint64_t seq_split_length;
-};
 
-typedef struct _stream_state stream_state_s;
 
-static void create_threads( pthread_t *threads, int num, void *(*start_routine)(void *) )
+void SeqEncoder::CreateThreads( pthread_t *threads,
+                                int num, void *(*start_routine)(void *),
+                                SeqEncoder *seqencoder )
 {
 	int i;
 	pthread_attr_t *pattr = NULL;
@@ -386,14 +243,14 @@ static void create_threads( pthread_t *threads, int num, void *(*start_routine)(
 #endif
 	for(i = 0; i < num; ++i )
 	{
-		if( pthread_create( &threads[i], pattr, start_routine, NULL ) != 0 )
+		if( pthread_create( &threads[i], pattr, start_routine, seqencoder ) != 0 )
 		{
 			mjpeg_error_exit1( "worker thread creation failed: %s", strerror(errno) );
 		}
 	}
 }
 
-static void gop_start( stream_state_s *ss )
+void SeqEncoder::GopStart( StreamState *ss )
 {
 
 	int nb, np;
@@ -408,7 +265,7 @@ static void gop_start( stream_state_s *ss )
 	ss->b = 0;
 	ss->new_seq = false;
 	
-	if( opt_pulldown_32 )
+	if( encparams.pulldown_32 )
 		frame_periods = (double)(ss->seq_start_frame + ss->i)*(5.0/4.0);
 	else
 		frame_periods = (double)(ss->seq_start_frame + ss->i);
@@ -418,16 +275,16 @@ static void gop_start( stream_state_s *ss )
       an estimate for the other streams based on time.
       For CBR we do *both* based on time to account for padding during
       muxing.
+      TODO: This should be placed in the bitrate controller....
     */
     
-    if( ctl_quant_floor > 0.0 )
-        bits_after_mux = bitcount() + 
-            (uint64_t)((frame_periods / opt_frame_rate) * ctl_nonvid_bit_rate);
+    if( encparams.quant_floor > 0.0 ) bits_after_mux = writer.BitCount() + 
+            (uint64_t)((frame_periods / encparams.frame_rate) * encparams.nonvid_bit_rate);
     else
-        bits_after_mux = (uint64_t)((frame_periods / opt_frame_rate) * 
-                                    (ctl_nonvid_bit_rate + opt_bit_rate));
+        bits_after_mux = (uint64_t)((frame_periods / encparams.frame_rate) * 
+                                    (encparams.nonvid_bit_rate + encparams.bit_rate));
 	if( (ss->next_split_point != 0ULL && bits_after_mux > ss->next_split_point)
-		|| (ss->i != 0 && opt_seq_end_every_gop)
+		|| (ss->i != 0 && encparams.seq_end_every_gop)
 		)
 	{
 		mjpeg_info( "Splitting sequence this GOP start" );
@@ -440,7 +297,7 @@ static void gop_start( stream_state_s *ss )
 		ss->new_seq = true;
 	}
 
-    ss->closed_gop = ss->i == 0 || ctl_closed_GOPs;
+    ss->closed_gop = ss->i == 0 || encparams.closed_GOPs;
 	ss->gop_start_frame = ss->seq_start_frame + ss->i;
 	
 	/*
@@ -452,10 +309,10 @@ static void gop_start( stream_state_s *ss )
 	*/
 	
     ss->gop_length =  
-        find_gop_length( ss->gop_start_frame, 
-                         ss->closed_gop ? 0 : ctl_M-1, 
-                         ctl_N_min, ctl_N_max,
-                         ctl_M_min);
+        FindGopLength( ss->gop_start_frame, 
+                       ss->closed_gop ? 0 : encparams.M-1, 
+                       encparams.N_min, encparams.N_max,
+                       encparams.M_min);
 			
 	/* First figure out how many B frames we're short from
 	   being able to achieve an even M-1 B's per I/P frame.
@@ -466,9 +323,11 @@ static void gop_start( stream_state_s *ss )
 	   A complication is the extra I-frame in the initial
 	   closed GOP of a sequence.
 	*/
-	if( ctl_M-1 > 0 )
+	if( encparams.M-1 > 0 )
 	{
-		ss->bs_short = (ctl_M - ((ss->gop_length-(ss->i==0)) % ctl_M))%ctl_M;
+        int pics_in_bigrps = 
+            ss->closed_gop ? ss->gop_length - 1 : ss->gop_length;
+		ss->bs_short = (encparams.M - pics_in_bigrps % encparams.M)%encparams.M;
 		ss->next_b_drop = ((double)ss->gop_length) / (double)(ss->bs_short+1)-1.0 ;
 	}
 	else
@@ -478,23 +337,17 @@ static void gop_start( stream_state_s *ss )
 	}
 	
 	/* We aim to spread the dropped B's evenly across the GOP */
-	ss->bigrp_length = (ctl_M-1);
+	ss->bigrp_length = (encparams.M-1);
 	
-	/* number of P frames */
-	if( ctl_M == 0 ) 
-	{
-		ss->bigrp_length = 0;
-		np = 0;
-	}
-	else if (ss->closed_gop )
+    if (ss->closed_gop )
 	{
 		ss->bigrp_length = 1;
-		np = (ss->gop_length + 2*(ctl_M-1))/ctl_M - 1; /* Closed GOP */
+		np = (ss->gop_length + 2*(encparams.M-1))/encparams.M - 1; /* Closed GOP */
 	}
 	else
 	{
-		ss->bigrp_length = ctl_M;
-		np = (ss->gop_length + (ctl_M-1))/ctl_M - 1;
+		ss->bigrp_length = encparams.M;
+		np = (ss->gop_length + (encparams.M-1))/encparams.M - 1;
 	}
 	/* number of B frames */
 	nb = ss->gop_length - np - 1;
@@ -510,73 +363,11 @@ static void gop_start( stream_state_s *ss )
 }
 
 
-
-/* Set the sequencing structure information
-   of a picture (type and temporal reference)
-   based on the specified sequence state
-*/
-
-static void I_or_P_frame_struct( stream_state_s *ss,
-								 Picture *picture )
-{
-	/* Temp ref of I frame in closed GOP of sequence is 0 We have to
-	   be a little careful with the end of stream special-case.
-	*/
-	if( ss->g == 0 && ss->closed_gop )
-	{
-		picture->temp_ref = 0;
-	}
-	else 
-	{
-		picture->temp_ref = ss->g+(ss->bigrp_length-1);
-	}
-
-	if (picture->temp_ref >= (istrm_nframes-ss->gop_start_frame))
-		picture->temp_ref = (istrm_nframes-ss->gop_start_frame) - 1;
-
-	picture->present = (ss->i-ss->g)+picture->temp_ref;
-	if (ss->g==0) /* first displayed frame in GOP is I */
-	{
-		picture->pict_type = I_TYPE;
-	}
-	else 
-	{
-		picture->pict_type = P_TYPE;
-	}
-
-	/* Start of GOP - set GOP data for picture */
-	if( ss->g == 0 )
-	{
-		picture->gop_start = true;
-        picture->closed_gop = ss->closed_gop;
-		picture->new_seq = ss->new_seq;
-		picture->nb = ss->nb;
-		picture->np = ss->np;
-	}		
-	else
-	{
-		picture->gop_start = false;
-        picture->closed_gop = false;
-		picture->new_seq = false;
-	}
-}
-
-
-static void B_frame_struct(  stream_state_s *ss,
-							 Picture *picture )
-{
-	picture->temp_ref = ss->g - 1;
-	picture->present = ss->i-1;
-	picture->pict_type = B_TYPE;
-	picture->gop_start = false;
-	picture->new_seq = false;
-}
-
 /*
   Update ss to the next sequence state.
 */
 
-static void next_seq_state( stream_state_s *ss )
+void SeqEncoder::NextSeqState( StreamState *ss )
 {
 	++(ss->i);
 	++(ss->g);
@@ -590,66 +381,24 @@ static void next_seq_state( stream_state_s *ss )
 		   come out right ? */
 		if( ss->bs_short != 0 && ss->g > (int)ss->next_b_drop )
 		{
-			ss->bigrp_length = ctl_M - 1;
+			ss->bigrp_length = encparams.M - 1;
 			if( ss->bs_short )
 				ss->next_b_drop += ((double)ss->gop_length) / (double)(ss->bs_short+1) ;
 		}
 		else
-			ss->bigrp_length = ctl_M;
+			ss->bigrp_length = encparams.M;
 	}
 
     /* Are we starting a new GOP? */
 	if( ss->g == ss->gop_length )
 	{
-		gop_start(ss);
+		GopStart( ss);
 	}
 
 }
 
 
 
-static void init_pict_data( Picture *picture )
-{
-	int i,j;
-	/* Allocate buffers for picture transformation */
-	picture->blocks = 
-        static_cast<DCTblock*>(
-            bufalloc(mb_per_pict*block_count*sizeof(DCTblock)));
-	picture->qblocks =
-		static_cast<DCTblock *>(
-            bufalloc(mb_per_pict*block_count*sizeof(DCTblock)));
-    DCTblock *blocks = picture->blocks;
-    DCTblock *qblocks = picture->qblocks;
-    for (j=0; j<opt_enc_height2; j+=16)
-    {
-        for (i=0; i<opt_enc_width; i+=16)
-        {
-            picture->mbinfo.push_back(MacroBlock(*picture, i,j, 
-                                                 blocks,qblocks ));
-            blocks += block_count;
-            qblocks += block_count;
-        }
-    }
-
-
-	picture->curref = new (uint8_t *)[3];
-	picture->curorg = new (uint8_t *)[3];
-	picture->pred   = new (uint8_t *)[3];
-
-	for( i = 0 ; i<3; i++)
-	{
-		int size =  (i==0) ? lum_buffer_size : chrom_buffer_size;
-		picture->curref[i] = static_cast<uint8_t *>(bufalloc(size));
-		picture->curorg[i] = NULL;
-		picture->pred[i]   = static_cast<uint8_t *>(bufalloc(size));
-	}
-
-	/* The (non-existent) previous encoding using an as-yet un-used
-	   picture encoding data buffers is "completed"
-	*/
-	sync_guard_init( &picture->completion, 1 );
-}
-
 
 /* 
    Initialize picture data buffers for reference and B pictures
@@ -667,61 +416,27 @@ static void init_pict_data( Picture *picture )
    more than M threads.   
 */
 
-#define R_PICS (MAX_WORKER_THREADS+2)
-#define B_PICS (MAX_WORKER_THREADS+2)
 
-static void init_pictures( Picture *ref_pictures, Picture *b_pictures )
+void SeqEncoder::LinkPictures( Picture *ref_pictures[], 
+                               Picture *b_pictures[] )
 {
 
 	int i,j;
-	
-	for( i = 0; i < R_PICS; ++i )
-		init_pict_data( &ref_pictures[i] );
 
-	for( i = 0; i < R_PICS; ++i )
+	for( i = 0; i < encparams.max_active_ref_frames; ++i )
 	{
-		j = (i + 1) % R_PICS;
+		j = (i + 1) % encparams.max_active_ref_frames;
 
-		ref_pictures[j].oldorg = ref_pictures[i].curorg;
-		ref_pictures[j].oldref = ref_pictures[i].curref;
-		ref_pictures[j].neworg = ref_pictures[j].curorg;
-		ref_pictures[j].newref = ref_pictures[j].curref;
+		ref_pictures[j]->oldorg = ref_pictures[i]->curorg;
+		ref_pictures[j]->oldref = ref_pictures[i]->curref;
+		ref_pictures[j]->neworg = ref_pictures[j]->curorg;
+		ref_pictures[j]->newref = ref_pictures[j]->curref;
 	}
 
-	for( i = 0; i < B_PICS; ++i )
-	{
-		init_pict_data( &b_pictures[i]);
-	}
 }
 
-/*
- *
- * Reconstruct the decoded image for references images and
- * for statistics
- *
- */
 
 
-static void reconstruct( Picture *picture)
-{
-
-#ifndef OUTPUT_STAT
-	if( picture->pict_type!=B_TYPE)
-	{
-#endif
-		iquantize( picture );
-		itransform(picture);
-		calcSNR(picture);
-		stats();
-#ifndef OUTPUT_STAT
-	}
-#endif
-}
-
-static mp_semaphore_t worker_available =  SEMAPHORE_INITIALIZER;
-static mp_semaphore_t picture_available = SEMAPHORE_INITIALIZER;
-static mp_semaphore_t picture_started = SEMAPHORE_INITIALIZER;
-
 #ifdef DEBUG_BLOCK_STRIPED
 static unsigned int checksum( uint8_t *buf, unsigned int len )
 {
@@ -733,24 +448,9 @@ static unsigned int checksum( uint8_t *buf, unsigned int len )
 }
 #endif
 
-static void encodembs(Picture *picture)
-{ 
-    vector<MacroBlock>::iterator mbi = picture->mbinfo.begin();
 
-	for( mbi = picture->mbinfo.begin(); mbi < picture->mbinfo.end(); ++mbi)
-	{
-        mbi->MotionEstimate();
-        // Temporary eventually we will select by
-        // a quick prediction and complexity estimate...
-        mbi->Predict();
-        mbi->Transform();
-	}
-
-}
-
-static void stencodeworker(Picture *picture)
+void SeqEncoder::SequentialEncode(Picture *picture)
 {
-	/* ALWAYS do-able */
 	mjpeg_debug("Frame start %d %c %d %d",
 			   picture->decode, 
 			   pict_type_char[picture->pict_type],
@@ -759,58 +459,35 @@ static void stencodeworker(Picture *picture)
 
 
 	if( picture->pict_struct != FRAME_PICTURE )
-		mjpeg_info("Field %s (%d)",
+		mjpeg_debug("Field %s (%d)",
 				   (picture->pict_struct == TOP_FIELD) ? "top" : "bot",
 				   picture->pict_struct
 			);
 
-	motion_subsampled_lum(picture);
+	picture->MotionSubSampledLum();
 
 		
-	/* DEPEND on completion previous Reference frames (P) or on old
-	   and new Reference frames B.  However, since new reference frame
-	   cannot complete until old reference frame completed (see below)
-	   suffices just to check new reference frame... 
-	   N.b. completion guard of picture is always reset to false
-	   before this function is called...
-		   
-	   In field picture encoding the P field of an I frame is
-	   a special case.  We have to wait for completion of the I field
-	   before starting the P field
-	*/
-
-#ifdef ORIGINAL_PHASE_BASED_PROC
-	motion_estimation(picture);
-	predict(picture);
-	transform(picture);
-#else
-    encodembs(picture);
-#endif
-	/* Depends on previous frame completion for IB and P */
-
-	picture->PutHeadersAndEncoding(*bitrate_controller);
-
-	reconstruct(picture);
+    picture->EncodeMacroBlocks();
+	picture->PutHeadersAndEncoding(ratecontroller);
+	picture->Reconstruct();
 
 	/* Handle second field of a frame that is being field encoded */
-	if( opt_fieldpic )
+	if( encparams.fieldpic )
 	{
-		set_2nd_field_params(picture);
-		mjpeg_info("Field %s (%d)",
+		picture->Set2ndField();
+		mjpeg_debug("Field %s (%d)",
 				   (picture->pict_struct == TOP_FIELD) ? "top" : "bot",
 				   picture->pict_struct
 			);
 
-		motion_estimation(picture);
-		predict(picture);
-		transform(picture);
-		picture->PutHeadersAndEncoding(*bitrate_controller);;
-		reconstruct(picture);
+        picture->EncodeMacroBlocks();
+		picture->PutHeadersAndEncoding(ratecontroller);
+		picture->Reconstruct();
 
 	}
 
 
-	mjpeg_info("Frame end %d %c quant=%3.2f total act=%8.1f %s", 
+	mjpeg_info("Frame end %d %c quant=%3.2f total act=%8.5f %s", 
                picture->decode, 
 			   pict_type_char[picture->pict_type],
                picture->AQ,
@@ -820,18 +497,15 @@ static void stencodeworker(Picture *picture)
 			
 }
 
-/*
-  Ohh, lovely C type syntax... more or less have to introduce a named
-  typed here to bind the "volatile" correctly - to the pointer not the
-  data it points to. K&R: hang your heads in shame...
-*/
-
-typedef Picture * pict_data_ptr;
-
-volatile static pict_data_ptr picture_to_encode;
 
 
-static void *parencodeworker(void *start_arg)
+void *SeqEncoder::ParallelEncodeWrapper(void *seqencoder)
+{
+    static_cast<SeqEncoder *>(seqencoder)->ParallelEncodeWorker();
+    return 0;
+}
+
+void SeqEncoder::ParallelEncodeWorker()
 {
 	pict_data_ptr picture;
 	mjpeg_debug( "Worker thread started" );
@@ -842,22 +516,23 @@ static void *parencodeworker(void *start_arg)
 		mp_semaphore_wait( &picture_available );
 		/* Precisely *one* worker is started after update of
 		   picture_for_started_worker, so no need for handshake.  */
-		picture = (Picture *)picture_to_encode;
+		picture = picture_to_encode;
 		mp_semaphore_signal( &picture_started, 1);
 
 		/* ALWAYS do-able */
-		mjpeg_info("Frame %d %c %d %d",  
-				   picture->decode,  pict_type_char[picture->pict_type],
-				   picture->temp_ref,
-				   picture->present);
+		mjpeg_debug("Frame %d  %c %d %d",  
+                    picture->decode,  
+                    pict_type_char[picture->pict_type],
+                    picture->temp_ref,
+                    picture->present);
 
 		if( picture->pict_struct != FRAME_PICTURE )
-			mjpeg_info("Field %s (%d)",
+			mjpeg_debug("Field %s (%d)",
 					   (picture->pict_struct == TOP_FIELD) ? "top" : "bot",
 					   picture->pict_struct
 				);
 
-		motion_subsampled_lum(picture);
+		picture->MotionSubSampledLum();
 
 		
 		/* DEPEND on completion previous Reference frames (P) or on old
@@ -871,66 +546,63 @@ static void *parencodeworker(void *start_arg)
 		   a special case.  We have to wait for completion of the I field
 		   before starting the P field
 		*/
-#ifdef ORIGINAL_PHASE_BASED_PROC
-		if( ctl_refine_from_rec )
-		{
-			sync_guard_test( picture->ref_frame_completion );
-			motion_estimation(picture);
-		}
-		else
-		{
-			motion_estimation(picture);
-			sync_guard_test( picture->ref_frame_completion );
-		}
-		predict(picture);
 
-		/* No dependency */
-		transform(picture);
-#else
-        sync_guard_test( picture->ref_frame_completion );
-        encodembs(picture);
+#ifdef SEQ_DEBUG
+        printf( "Frame %d %08x Waiting for ref: %d %08x\n",
+                picture->decode, 
+                picture, 
+                picture->ref_frame->decode,
+                picture->ref_frame );
 #endif
+        sync_guard_test( &picture->ref_frame->completion );
+        picture->EncodeMacroBlocks();
+
 		/* Depends on previous frame completion for IB and P */
-		sync_guard_test( picture->prev_frame_completion );
-		picture->PutHeadersAndEncoding(*bitrate_controller);
+#ifdef SEQ_DEBUG
+        printf( "Frame %d %08x Waiting for compl: %d %08x\n",
+                picture->decode, 
+                picture, 
+                picture->prev_frame->decode,
+                picture->prev_frame );
+#endif
+		sync_guard_test( &picture->prev_frame->completion );
+		picture->PutHeadersAndEncoding(ratecontroller);
 
-		reconstruct(picture);
+		picture->Reconstruct();
 		/* Handle second field of a frame that is being field encoded */
-		if( opt_fieldpic )
+		if( encparams.fieldpic )
 		{
-			set_2nd_field_params(picture);
+			picture->Set2ndField();
 
-			mjpeg_info("Field %s (%d)",
-					   (picture->pict_struct == TOP_FIELD) ? "top" : "bot",
-					   picture->pict_struct
+			mjpeg_debug("Field %s (%d)",
+                        (picture->pict_struct == TOP_FIELD) ? "top" : "bot",
+                        picture->pict_struct
 				);
 
-			motion_estimation(picture);
-			predict(picture);
-			transform(picture);
-            picture->PutHeadersAndEncoding(*bitrate_controller);
-			reconstruct(picture);
+            picture->EncodeMacroBlocks();
+            picture->PutHeadersAndEncoding(ratecontroller);
+			picture->Reconstruct();
 
 		}
 
 
-		mjpeg_info("Frame end %d %s %3.2f %.2f %2.1f %.2f",
-					picture->decode, 
-					picture->pad ? "PAD" : "   ",
-					picture->avg_act, picture->sum_avg_act,
-					picture->AQ, picture->SQ);
+	mjpeg_info("Frame end %d %c quant=%3.2f total act=%8.5f %s", 
+               picture->decode, 
+			   pict_type_char[picture->pict_type],
+               picture->AQ,
+               picture->sum_avg_act,
+               picture->pad ? "PAD" : "   "
+        );
 
 		/* We're finished - let anyone depending on us know...
 		 */
 		sync_guard_update( &picture->completion, 1 );
 			
 	}
-
-	return NULL;
 }
 
 
-static void parencodepict( Picture *picture )
+void SeqEncoder::ParallelEncode( Picture *picture )
 {
 
 	mp_semaphore_wait( &worker_available );
@@ -951,40 +623,52 @@ static void parencodepict( Picture *picture )
  *  Picture record for the oldest frame that could still be needed
  *  by active worker threads.
  * 
- *  Buffers sizes are given by R_PICS and B_PICS
+ *  Buffers sizes are given by encparams.max_active_ref_frames and
+ *  encparams.max_active_b_frames
  *
  ********************/
  
-void putseq(void)
+void SeqEncoder::Encode()
 {
-    OnTheFlyRateCtl ratectl;
-    bitrate_controller = &ratectl;
     /* DEBUG */
 	uint64_t bits_after_mux;
 	double frame_periods;
     /* END DEBUG */
-	stream_state_s ss;
+	StreamState ss;
 	int cur_ref_idx = 0;
 	int cur_b_idx = 0;
-	Picture b_pictures[B_PICS];
-	Picture ref_pictures[R_PICS];
-	pthread_t worker_threads[MAX_WORKER_THREADS];
-	Picture *cur_picture, *old_picture;
-	Picture *new_ref_picture, *old_ref_picture;
 
-	init_pictures( ref_pictures, b_pictures );
-	if( ctl_max_encoding_frames > 1 )
-		create_threads( worker_threads, 2, parencodeworker );
+    int i;
+	Picture *b_pictures[encparams.max_active_b_frames];
+	Picture *ref_pictures[encparams.max_active_ref_frames];
+    for( i = 0; i < encparams.max_active_b_frames; ++i )
+    {
+        b_pictures[i] = new Picture(encparams, coder, quantizer);
+    }
+    for( i = 0; i < encparams.max_active_ref_frames; ++i )
+    {
+        ref_pictures[i] = new Picture(encparams, coder, quantizer);
+    }
+
+	LinkPictures( ref_pictures, b_pictures );
+
+	pthread_t worker_threads[encparams.max_encoding_frames];
+	if( encparams.max_encoding_frames > 1 )
+		CreateThreads( worker_threads, encparams.max_encoding_frames, 
+                       ParallelEncodeWrapper, this );
 
 	/* Initialize image dependencies and synchronisation.  The
 	   first frame encoded has no predecessor whose completion it
 	   must wait on.
 	*/
-	old_ref_picture = &ref_pictures[R_PICS-1];
-	new_ref_picture = &ref_pictures[cur_ref_idx];
-	cur_picture = old_ref_picture;
+	Picture *cur_picture, *old_picture;
+	Picture *new_ref_picture, *old_ref_picture;
+
+	old_ref_picture = ref_pictures[encparams.max_active_ref_frames-1];
+	new_ref_picture = ref_pictures[cur_ref_idx];
+	cur_picture = new_ref_picture;
 	
-	ratectl.InitSeq(false);
+	ratecontroller.InitSeq(false);
 	
 	ss.i = 0;		                /* Index in current MPEG sequence */
 	ss.g = 0;						/* Index in current GOP */
@@ -994,96 +678,113 @@ void putseq(void)
 	ss.seq_start_frame = 0;		/* Index start current sequence in
 								   input stream */
 	ss.gop_start_frame = 0;		/* Index start current gop in input stream */
-	ss.seq_split_length = ((int64_t)ctl_seq_length_limit)*(8*1024*1024);
+	ss.seq_split_length = ((int64_t)encparams.seq_length_limit)*(8*1024*1024);
 	ss.next_split_point = BITCOUNT_OFFSET + ss.seq_split_length;
 	mjpeg_debug( "Split len = %" PRId64 "", ss.seq_split_length );
 
-	frame_num = 0;              /* Encoding number */
+	int frame_num = 0;              /* Encoding number */
 
-	gop_start(&ss);
+	GopStart( &ss);
 
 	/* loop through all frames in encoding/decoding order */
-	while( frame_num<istrm_nframes )
+	while( frame_num< reader.NumberOfFrames() )
 	{
 		old_picture = cur_picture;
 
 		/* Each bigroup starts once all the B frames of its predecessor
 		   have finished.
 		*/
+        int index;
+        char type;
 		if ( ss.b == 0)
 		{
-			cur_ref_idx = (cur_ref_idx + 1) % R_PICS;
+            type = 'R';
+			cur_ref_idx = (cur_ref_idx + 1) % encparams.max_active_ref_frames;
+            index = cur_ref_idx;
 			old_ref_picture = new_ref_picture;
-			new_ref_picture = &ref_pictures[cur_ref_idx];
-			new_ref_picture->ref_frame_completion = &old_ref_picture->completion;
-			new_ref_picture->prev_frame_completion = &cur_picture->completion;
-			I_or_P_frame_struct(&ss, new_ref_picture);
+			new_ref_picture = ref_pictures[cur_ref_idx];
+			new_ref_picture->ref_frame = old_ref_picture;
+			new_ref_picture->prev_frame = cur_picture;
+			new_ref_picture->Set_IP_Frame(&ss, reader.NumberOfFrames());
 			cur_picture = new_ref_picture;
 		}
 		else
 		{
+            type = 'B';
+
 			Picture *new_b_picture;
 			/* B frame: no need to change the reference frames.
 			   The current frame data pointers are a 3rd set
 			   seperate from the reference data pointers.
 			*/
-			cur_b_idx = ( cur_b_idx + 1) % B_PICS;
-			new_b_picture = &b_pictures[cur_b_idx];
+			cur_b_idx = ( cur_b_idx + 1) % encparams.max_active_b_frames;
+            index = cur_b_idx;
+			new_b_picture = b_pictures[cur_b_idx];
 			new_b_picture->oldorg = new_ref_picture->oldorg;
 			new_b_picture->oldref = new_ref_picture->oldref;
 			new_b_picture->neworg = new_ref_picture->neworg;
 			new_b_picture->newref = new_ref_picture->newref;
-			new_b_picture->ref_frame_completion = &new_ref_picture->completion;
-			new_b_picture->prev_frame_completion = &cur_picture->completion;
-			B_frame_struct( &ss, new_b_picture );
+			new_b_picture->ref_frame = new_ref_picture;
+			new_b_picture->prev_frame = cur_picture;
+			new_b_picture->Set_B_Frame( &ss );
 			cur_picture = new_b_picture;
 		}
 
+#ifdef SEQ_DEBUG
+        printf( "Mark incomplete: %d %08x prev = %08x ref = %08x\n", index, cur_picture, cur_picture->ref_frame, cur_picture->prev_frame );
+#endif
 		sync_guard_update( &cur_picture->completion, 0 );
+		reader.ReadFrame( cur_picture->temp_ref+ss.gop_start_frame,
+                          cur_picture->curorg );
 
-		if( readframe(cur_picture->temp_ref+ss.gop_start_frame,cur_picture->curorg) )
-		{
-		    mjpeg_error_exit1("Corrupt frame data in frame %d aborting!",
-							  cur_picture->temp_ref+ss.gop_start_frame );
-		}
-
-
-		set_pic_params( ss.i, ss.b,   cur_picture );
-		if( ctl_max_encoding_frames > 1 )
-			parencodepict( cur_picture );
+		cur_picture->SetSeqPos( ss.i, ss.b );
+		if( encparams.max_encoding_frames > 1 )
+			ParallelEncode( cur_picture );
 		else
-			stencodeworker( cur_picture );
+			SequentialEncode( cur_picture );
 
 #ifdef DEBUG
 		writeframe(cur_picture->temp_ref+ss.gop_start_frame,cur_picture->curref);
 #endif
 
-		next_seq_state( &ss );
+		NextSeqState( &ss );
 		++frame_num;
 	}
 	
 	/* Wait for final frame's encoding to complete */
-	if( ctl_max_encoding_frames > 1 )
+	if( encparams.max_encoding_frames > 1 )
 		sync_guard_test( &cur_picture->completion );
-	putseqend();
+	coder.PutSeqEnd();
 
-	if( opt_pulldown_32 )
+	if( encparams.pulldown_32 )
 		frame_periods = (double)(ss.seq_start_frame + ss.i)*(5.0/4.0);
 	else
 		frame_periods = (double)(ss.seq_start_frame + ss.i);
     
-    /* DEBUG */
     
-    if( ctl_quant_floor > 0.0 )
-        bits_after_mux = bitcount() + 
-            (uint64_t)((frame_periods / opt_frame_rate) * ctl_nonvid_bit_rate);
+    // TODO Belongs in BitRateCtl
+    if( encparams.quant_floor > 0.0 )
+        bits_after_mux = writer.BitCount() + 
+            (uint64_t)((frame_periods / encparams.frame_rate) * encparams.nonvid_bit_rate);
     else
-        bits_after_mux = (uint64_t)((frame_periods / opt_frame_rate) * 
-                                    (ctl_nonvid_bit_rate + opt_bit_rate));
+        bits_after_mux = (uint64_t)((frame_periods / encparams.frame_rate) * 
+                                    (encparams.nonvid_bit_rate + encparams.bit_rate));
 
     mjpeg_info( "Guesstimated final muxed size = %lld\n", bits_after_mux/8 );
-    /* END DEBUG */
-
+    
+    //
+    // TODO: Really this ought to be a job for smart ptrs, but auto_ptr
+    // gets messed with the copy constructor implicit in push_back...
+    //
+    for( i = 0; i < encparams.max_active_b_frames; ++i )
+    {
+        delete b_pictures[i];
+    }
+    for( i = 0; i < encparams.max_active_ref_frames; ++i )
+    {
+        delete ref_pictures[i];
+    }
+    
 }
 
 
