@@ -21,7 +21,6 @@
 #ifndef _CPU_ACCEL_HH
 #define _CPU_ACCEL_HH
 
-#include <config.h>
 #include <stdlib.h>
 #include "mjpeg_types.h"
 
@@ -41,6 +40,9 @@ extern "C" {
 #if	!defined(HAVE_POSIX_MEMALIGN)
 	int posix_memalign(void **, size_t, size_t);
 #endif
+#if	!defined(HAVE_MEMALIGN)
+	void *memalign(size_t, size_t);
+#endif
 #if	!defined(HAVE_FMAX)
 	double fmax(double, double);
 #endif
