@@ -91,17 +91,19 @@ struct DNSR_GLOBAL
     {
       uint16_t  x;
       uint16_t  y;
-      uint16_t  w;
-      uint16_t  h;      
+      int16_t   w;
+      int16_t   h;      
     } border;
     
   };
   
 struct DNSR_VECTOR
 {
-  int8_t  x;
-  int8_t  y;
+  int8_t     x;
+  int8_t     y;
   uint32_t SAD;
 };
 
+uint8_t luma_contrast_vector[256];
+
 #endif
